@@ -27,6 +27,16 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: channel.h,v $
+ * Revision 1.42  2003/12/19 04:29:52  csoutheren
+ * Changed GetLastReadCount and GetLastWriteCount to be virtual
+ *
+ * Revision 1.41  2003/09/17 05:41:58  csoutheren
+ * Removed recursive includes
+ *
+ * Revision 1.40  2003/09/17 01:18:02  csoutheren
+ * Removed recursive include file system and removed all references
+ * to deprecated coooperative threading support
+ *
  * Revision 1.39  2002/09/16 01:08:59  robertj
  * Added #define so can select if #pragma interface/implementation is used on
  *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
@@ -156,14 +166,14 @@
  *
  */
 
-
+#ifndef _PCHANNEL
 #define _PCHANNEL
 
 #ifdef P_USE_PRAGMA
 #pragma interface
 #endif
 
-
+#include <ptlib/mutex.h>
 
 ///////////////////////////////////////////////////////////////////////////////
 // I/O Channels
@@ -376,7 +386,7 @@ class PChannel : public PObject, public iostream {
        @return
        the number of bytes read.
      */
-    PINDEX GetLastReadCount() const;
+    virtual PINDEX GetLastReadCount() const;
 
     /** Read a single 8 bit byte from the channel. If one was not available
        within the read timeout period, or an I/O error occurred, then the
@@ -485,7 +495,7 @@ class PChannel : public PObject, public iostream {
        @return
        the number of bytes written.
      */
-    PINDEX GetLastWriteCount() const;
+    virtual PINDEX GetLastWriteCount() const;
 
     /** Write a single character to the channel. This function simply uses the
        Write() function so all comments on that function also apply.
@@ -788,8 +798,14 @@ class PChannel : public PObject, public iostream {
 
 
 // Include platform dependent part of class
-#include <ptlib/channel.h>
+#ifdef _WIN32
+#include "msos/ptlib/channel.h"
+#else
+#include "unix/ptlib/channel.h"
+#endif
+
 };
 
+#endif
 
 // End Of File ///////////////////////////////////////////////////////////////
