@@ -27,6 +27,16 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: socket.h,v $
+ * Revision 1.45  2004/01/28 08:53:02  csoutheren
+ * Fixed missing delete[] operator. Thanks to Borko Jandras
+ *
+ * Revision 1.44  2003/09/17 05:41:59  csoutheren
+ * Removed recursive includes
+ *
+ * Revision 1.43  2003/09/17 01:18:02  csoutheren
+ * Removed recursive include file system and removed all references
+ * to deprecated coooperative threading support
+ *
  * Revision 1.42  2002/10/18 08:07:41  robertj
  * Fixed use of FD_ZERO as (strangely) crashes on some paltforms and would
  *   not have cleared enough of an enlarges fd_set anyway.
@@ -162,14 +172,14 @@
  *
  */
 
+#ifndef _PSOCKETS
+#define _PSOCKETS
+
 #ifdef P_USE_PRAGMA
 #pragma interface
 #endif
 
-
-#ifndef _PCHANNEL
 #include <ptlib/channel.h>
-#endif
 
 #ifdef __NUCLEUS_PLUS__
 #include <sys/socket.h>
@@ -582,9 +592,12 @@ class PSocket : public PChannel
     /// Port to be used by the socket when opening the channel.
     WORD port;
 
-
 // Include platform dependent part of class
-#include <ptlib/socket.h>
+#ifdef _WIN32
+#include "msos/ptlib/socket.h"
+#else
+#include "unix/ptlib/socket.h"
+#endif
 };
 
 
@@ -596,7 +609,7 @@ class P_fd_set {
     P_fd_set(SOCKET fd);
     ~P_fd_set()
       {
-        delete set;
+        delete[] set;
       }
 
     P_fd_set & operator=(SOCKET fd);
@@ -653,6 +666,20 @@ class P_timeval {
     BOOL infinite;
 };
 
+#ifdef _WIN32
+class PWinSock : public PSocket
+{
+  PCLASSINFO(PWinSock, PSocket)
+// Must be one and one only instance of this class, and it must be static!.
+  public:
+    PWinSock();
+    ~PWinSock();
+  private:
+    virtual BOOL OpenSocket();
+    virtual const char * GetProtocolName() const;
+};
+#endif
 
+#endif
 
 // End Of File ///////////////////////////////////////////////////////////////
