@@ -29,7 +29,7 @@
  * $log$
  */
 
-#if !defined(__BEOS__) && !defined(P_VXWORKS) && !defined(P_RTEMS)
+#if !defined(P_VXWORKS) && !defined(P_RTEMS)
 
 ///////////////////////////////////////////////////////////////////////////////
 // PDynaLink
@@ -37,7 +37,7 @@
   protected:
     void * dllHandle;
 
-#endif // !__BEOS__ && !P_VXWORKS
+#endif // !P_VXWORKS && !P_RTEMS
 
 
 // End Of File ////////////////////////////////////////////////////////////////
