@@ -27,6 +27,18 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: pmachdep.h,v $
+ * Revision 1.61  2004/02/23 20:00:15  ykiryanov
+ * Fixed a typo in declaration of setegid(0
+ *
+ * Revision 1.60  2004/02/22 04:33:19  ykiryanov
+ * Added missing prototype for setegid for BeOS
+ *
+ * Revision 1.59  2004/02/22 03:27:30  ykiryanov
+ * Added missing prototype for seteuid for BeOS
+ *
+ * Revision 1.58  2004/02/21 21:26:30  ykiryanov
+ * Added P_THREADIDENTIFIER for BeOS threads
+ *
  * Revision 1.57  2003/09/17 01:18:03  csoutheren
  * Removed recursive include file system and removed all references
  * to deprecated coooperative threading support
@@ -405,6 +417,8 @@ struct servent * getservbyname(const char *, const char *);
 typedef int socklen_t;
 #define wait3(s, o, r) waitpid(-1, s, o)
 #define PSETPGRP()  setpgid(0,0)
+int seteuid(uid_t euid);
+int setegid(gid_t gid);
 
 #ifndef BE_BONELESS
 #include <bone/arpa/inet.h>
@@ -647,6 +661,10 @@ typedef	int SOCKET;
 
 #endif  // P_PTHREADS
 
+#ifdef BE_THREADS
+#define P_THREADIDENTIFIER thread_id
+#endif // BE_THREADS
+
 #endif // _PMACHDEP_H
 
 // End of file
