@@ -24,6 +24,13 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: pstun.cxx,v $
+ * Revision 1.13  2004/03/14 05:47:52  rjongbloed
+ * Fixed incorrect detection of symmetric NAT (eg Linux masquerading) and also
+ *   some NAT systems which are partially blocked due to firewall rules.
+ *
+ * Revision 1.12  2004/02/24 11:15:48  rjongbloed
+ * Added function to get external router address, also did a bunch of documentation.
+ *
  * Revision 1.11  2004/02/17 11:11:05  rjongbloed
  * Added missing #pragma pack() to turn off byte alignment for the last class, thanks Ted Szoczei
  *
@@ -72,7 +79,9 @@
 PSTUNClient::PSTUNClient(const PString & server,
                          WORD portBase, WORD portMax,
                          WORD portPairBase, WORD portPairMax)
-  : serverAddress(0)
+  : serverAddress(0),
+    cachedExternalAddress(0),
+    timeAddressObtained(0)
 {
   serverPort = DefaultPort;
   Construct();
@@ -86,7 +95,9 @@ PSTUNClient::PSTUNClient(const PIPSocket::Address & address, WORD port,
                          WORD portBase, WORD portMax,
                          WORD portPairBase, WORD portPairMax)
   : serverAddress(address),
-    serverPort(port)
+    serverPort(port),
+    cachedExternalAddress(0),
+    timeAddressObtained(0)
 {
   Construct();
   SetPortRanges(portBase, portMax, portPairBase, portPairMax);
@@ -455,14 +466,15 @@ PSTUNClient::NatTypes PSTUNClient::GetNatType(BOOL force)
 
   // RFC3489 discovery
 
-  // test I
+  /* test I - the client sends a STUN Binding Request to a server, without
+     any flags set in the CHANGE-REQUEST attribute, and without the
+     RESPONSE-ADDRESS attribute. This causes the server to send the response
+     back to the address and port that the request came from. */
   PSTUNMessage requestI(PSTUNMessage::BindingRequest);
   requestI.AddAttribute(PSTUNChangeRequest(false, false));
   PSTUNMessage responseI;
-  if (!responseI.Poll(socket, requestI))
-  {
-    if (socket.GetErrorCode(PChannel::LastWriteError) != PChannel::NoError)
-    {
+  if (!responseI.Poll(socket, requestI)) {
+    if (socket.GetErrorCode(PChannel::LastWriteError) != PChannel::NoError) {
       PTRACE(1, "STUN\tError writing to server " << serverAddress << ':' << serverPort << " - " << socket.GetErrorText(PChannel::LastWriteError));
       return natType = UnknownNat; // No response usually means blocked
     }
@@ -472,8 +484,7 @@ PSTUNClient::NatTypes PSTUNClient::GetNatType(BOOL force)
   }
 
   PSTUNMappedAddress * mappedAddress = (PSTUNMappedAddress *)responseI.FindAttribute(PSTUNAttribute::MAPPED_ADDRESS);
-  if (mappedAddress == NULL)
-  {
+  if (mappedAddress == NULL) {
     PTRACE(2, "STUN\tExpected mapped address attribute from server " << serverAddress << ':' << serverPort);
     return natType = UnknownNat; // Protocol error
   }
@@ -482,13 +493,14 @@ PSTUNClient::NatTypes PSTUNClient::GetNatType(BOOL force)
   WORD mappedPortI = mappedAddress->port;
   bool notNAT = socket.GetPort() == mappedPortI && PIPSocket::IsLocalHost(mappedAddressI);
 
+  /* Test II - the client sends a Binding Request with both the "change IP"
+     and "change port" flags from the CHANGE-REQUEST attribute set. */
   PSTUNMessage requestII(PSTUNMessage::BindingRequest);
   requestII.AddAttribute(PSTUNChangeRequest(true, true));
   PSTUNMessage responseII;
   bool testII = responseII.Poll(socket, requestII);
 
-  if (notNAT)
-  {
+  if (notNAT) {
     // Is not NAT or symmetric firewall
     return natType = (testII ? OpenNat : SymmetricFirewall);
   }
@@ -500,29 +512,29 @@ PSTUNClient::NatTypes PSTUNClient::GetNatType(BOOL force)
   if (changedAddress == NULL)
     return natType = UnknownNat; // Protocol error
 
-  // Send to another server
-  bool mappedIPdifferent = false;
+  // Send test I to another server, to see if restricted or symmetric
   PIPSocket::Address secondaryServer = changedAddress->GetIP();
   WORD secondaryPort = changedAddress->port;
   socket.SetSendAddress(secondaryServer, secondaryPort);
   PSTUNMessage requestI2(PSTUNMessage::BindingRequest);
   requestI2.AddAttribute(PSTUNChangeRequest(false, false));
   PSTUNMessage responseI2;
-  if (responseI2.Poll(socket, requestI2))
-  {
-    mappedAddress = (PSTUNMappedAddress *)responseI.FindAttribute(PSTUNAttribute::MAPPED_ADDRESS);
-    if (mappedAddress == NULL)
-    {
-      PTRACE(2, "STUN\tExpected mapped address attribute from server " << serverAddress << ':' << serverPort);
-      return UnknownNat; // Protocol error
-    }
-
-    mappedIPdifferent = mappedAddress->port != mappedPortI || mappedAddress->GetIP() != mappedAddressI;
+  if (!responseI2.Poll(socket, requestI2)) {
+    PTRACE(2, "STUN\tPoll of secondary server " << secondaryServer << ':' << secondaryPort
+           << " failed, NAT partially blocked by firwall rules.");
+    return natType = PartialBlockedNat;
   }
 
-  if (mappedIPdifferent)
+  mappedAddress = (PSTUNMappedAddress *)responseI2.FindAttribute(PSTUNAttribute::MAPPED_ADDRESS);
+  if (mappedAddress == NULL) {
+    PTRACE(2, "STUN\tExpected mapped address attribute from server " << serverAddress << ':' << serverPort);
+    return UnknownNat; // Protocol error
+  }
+
+  if (mappedAddress->port != mappedPortI || mappedAddress->GetIP() != mappedAddressI)
     return natType = SymmetricNat;
 
+  socket.SetSendAddress(serverAddress, serverPort);
   PSTUNMessage requestIII(PSTUNMessage::BindingRequest);
   requestIII.SetAttribute(PSTUNChangeRequest(false, true));
   PSTUNMessage responseIII;
@@ -540,15 +552,55 @@ PString PSTUNClient::GetNatTypeName(BOOL force)
     "Port Restricted NAT",
     "Symmetric NAT",
     "Symmetric Firewall",
-    "Blocked"
+    "Blocked",
+    "Partially Blocked"
   };
 
   return Names[GetNatType(force)];
 }
 
 
+BOOL PSTUNClient::GetExternalAddress(PIPSocket::Address & externalAddress,
+                                     const PTimeInterval & maxAge)
+{
+  if (cachedExternalAddress.IsValid() && (PTime() - timeAddressObtained > maxAge)) {
+    externalAddress = cachedExternalAddress;
+    return TRUE;
+  }
+
+  externalAddress = 0; // Set to invalid address
+
+  PUDPSocket socket;
+  if (!OpenSocket(socket, singlePortInfo))
+    return false;
+
+  PSTUNMessage request(PSTUNMessage::BindingRequest);
+  request.AddAttribute(PSTUNChangeRequest(false, false));
+  PSTUNMessage response;
+  if (!response.Poll(socket, request))
+  {
+    PTRACE(1, "STUN\tServer " << serverAddress << ':' << serverPort << " unexpectedly went offline.");
+    return false;
+  }
+
+  PSTUNMappedAddress * mappedAddress = (PSTUNMappedAddress *)response.FindAttribute(PSTUNAttribute::MAPPED_ADDRESS);
+  if (mappedAddress == NULL)
+  {
+    PTRACE(2, "STUN\tExpected mapped address attribute from server " << serverAddress << ':' << serverPort);
+    return false;
+  }
+
+  
+  externalAddress = cachedExternalAddress = mappedAddress->GetIP();
+  timeAddressObtained = PTime();
+  return true;
+}
+
+
 BOOL PSTUNClient::CreateSocket(PUDPSocket * & socket)
 {
+  socket = NULL;
+
   switch (GetNatType(FALSE)) {
     case ConeNat :
     case RestrictedNat :
@@ -582,7 +634,8 @@ BOOL PSTUNClient::CreateSocket(PUDPSocket * & socket)
       if (mappedAddress != NULL)
       {
         stunSocket->externalIP = mappedAddress->GetIP();
-        stunSocket->port = mappedAddress->port;
+        if (GetNatType(FALSE) != SymmetricNat)
+          stunSocket->port = mappedAddress->port;
         stunSocket->SetSendAddress(0, 0);
         stunSocket->SetReadTimeout(PMaxTimeInterval);
         socket = stunSocket;
@@ -603,6 +656,9 @@ BOOL PSTUNClient::CreateSocket(PUDPSocket * & socket)
 BOOL PSTUNClient::CreateSocketPair(PUDPSocket * & socket1,
                                    PUDPSocket * & socket2)
 {
+  socket1 = NULL;
+  socket2 = NULL;
+
   switch (GetNatType(FALSE)) {
     case ConeNat :
     case RestrictedNat :
@@ -658,7 +714,8 @@ BOOL PSTUNClient::CreateSocketPair(PUDPSocket * & socket1,
       PTRACE(2, "STUN\tExpected mapped address attribute from server " << serverAddress << ':' << serverPort);
       return false;
     }
-    stunSocket[i].port = mappedAddress->port;
+    if (GetNatType(FALSE) != SymmetricNat)
+      stunSocket[i].port = mappedAddress->port;
     stunSocket[i].externalIP = mappedAddress->GetIP();
   }
 
