@@ -27,6 +27,28 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: collect.cxx,v $
+ * Revision 1.65  2004/02/15 03:04:52  rjongbloed
+ * Fixed problem with PSortedList nil variable and assignment between instances,
+ *   pointed out by Ben Lear.
+ *
+ * Revision 1.64  2004/02/09 06:23:32  csoutheren
+ * Added fix for gcc 3.3.1 problem. Apparently, it is unable to correctly resolve
+ * a function argument that is a reference to a const pointer. Changing the argument
+ * to be a pointer to a pointer solves the problem. Go figure
+ *
+ * Revision 1.63  2004/02/08 22:46:35  csoutheren
+ * Added casts to fix problems with gcc
+ *
+ * Revision 1.62  2004/02/08 11:13:20  rjongbloed
+ * Fixed crash in heavily loaded multi-threaded systems using simultaneous sorted
+ *   lists, Thanks Federico Pinna, Fabrizio Ammollo and the gang at Reitek S.p.A.
+ *
+ * Revision 1.61  2003/09/24 22:13:58  dereksmithies
+ * Add fix for deleting an object, then readding it. Thanks Fabrizio Ammollo
+ *
+ * Revision 1.60  2003/08/31 22:11:30  dereksmithies
+ * Fix from Diego Tartara for the SetAt function. Many thanks.
+ *
  * Revision 1.59  2002/12/02 04:27:37  robertj
  * Added extra bullet proofing for some pathological conditions.
  *
@@ -608,6 +630,18 @@ BOOL PAbstractList::SetAt(PINDEX index, PObject * val)
   return TRUE;
 }
 
+BOOL PAbstractList::ReplaceAt(PINDEX index, PObject * val)
+{
+  if (!SetCurrent(index))
+    return FALSE;
+  
+  if (info->lastElement->data != NULL && reference->deleteObjects) {
+    delete info->lastElement->data;
+  }
+
+  info->lastElement->data = val;
+  return TRUE;
+}
 
 PINDEX PAbstractList::GetObjectsIndex(const PObject * obj) const
 {
@@ -685,12 +719,17 @@ PAbstractList::Element::Element(PObject * theData)
 
 ///////////////////////////////////////////////////////////////////////////////
 
-PAbstractSortedList::Element PAbstractSortedList::Element::nil = NULL;
-
 PAbstractSortedList::PAbstractSortedList()
 {
   info = new Info;
   PAssert(info != NULL, POutOfMemory);
+  info->root = &info->nil;
+  info->lastElement = NULL;
+  info->lastIndex = P_MAX_INDEX;
+  info->nil.parent = info->nil.left = info->nil.right = &info->nil;
+  info->nil.subTreeSize = 0;
+  info->nil.colour = Element::Black;
+  info->nil.data = NULL;
 }
 
 
@@ -710,16 +749,16 @@ void PAbstractSortedList::CopyContents(const PAbstractSortedList & list)
 void PAbstractSortedList::CloneContents(const PAbstractSortedList * list)
 {
   Element * element = list->info->root;
-  while (element->left != &Element::nil)
+  while (element->left != &info->nil)
     element = element->left;
 
   info = new Info;
   PAssert(info != NULL, POutOfMemory);
   reference->size = 0;
 
-  while (element != &Element::nil) {
+  while (element != &info->nil) {
     Append(element->data->Clone());
-    element = element->Successor();
+    element = Successor(element);
   }
 }
 
@@ -734,24 +773,24 @@ PObject::Comparison PAbstractSortedList::Compare(const PObject & obj) const
 {
   PAssert(obj.IsDescendant(PAbstractSortedList::Class()), PInvalidCast);
   Element * elmt1 = info->root;
-  while (elmt1->left != &Element::nil)
+  while (elmt1->left != &info->nil)
     elmt1 = elmt1->left;
 
   Element * elmt2 = ((const PAbstractSortedList &)obj).info->root;
-  while (elmt2->left != &Element::nil)
+  while (elmt2->left != &info->nil)
     elmt2 = elmt2->left;
 
-  while (elmt1 != &Element::nil && elmt2 != &Element::nil) {
-    if (elmt1 == &Element::nil)
+  while (elmt1 != &info->nil && elmt2 != &info->nil) {
+    if (elmt1 == &info->nil)
       return LessThan;
-    if (elmt2 == &Element::nil)
+    if (elmt2 == &info->nil)
       return GreaterThan;
     if (*elmt1->data < *elmt2->data)
       return LessThan;
     if (*elmt1->data > *elmt2->data)
       return GreaterThan;
-    elmt1 = elmt1->Successor();
-    elmt2 = elmt2->Successor();
+    elmt1 = Successor(elmt1);
+    elmt2 = Successor(elmt2);
   }
   return EqualTo;
 }
@@ -759,16 +798,24 @@ PObject::Comparison PAbstractSortedList::Compare(const PObject & obj) const
 
 PINDEX PAbstractSortedList::Append(PObject * obj)
 {
-  Element * z = new Element(PAssertNULL(obj));
+  if (PAssertNULL(obj) == NULL)
+    return P_MAX_INDEX;
+
+  Element * z = new Element;
+  z->parent = z->left = z->right = &info->nil;
+  z->colour = Element::Black;
+  z->subTreeSize = 1;
+  z->data = obj;
+
   Element * x = info->root;
-  Element * y = &Element::nil;
-  while (x != &Element::nil) {
+  Element * y = &info->nil;
+  while (x != &info->nil) {
     x->subTreeSize++;
     y = x;
     x = *z->data < *x->data ? x->left : x->right;
   }
   z->parent = y;
-  if (y == &Element::nil)
+  if (y == &info->nil)
     info->root = z;
   else if (*z->data < *y->data)
     y->left = z;
@@ -844,8 +891,8 @@ BOOL PAbstractSortedList::Remove(const PObject * obj)
 
 PObject * PAbstractSortedList::RemoveAt(PINDEX index)
 {
-  Element * node = info->root->OrderSelect(index+1);
-  if (node == &Element::nil)
+  Element * node = OrderSelect(info->root, index+1);
+  if (node == &info->nil)
     return NULL;
 
   PObject * data = node->data;
@@ -856,10 +903,10 @@ PObject * PAbstractSortedList::RemoveAt(PINDEX index)
 
 void PAbstractSortedList::RemoveAll()
 {
-  if (info->root != &Element::nil) {
-    info->root->DeleteSubTrees(reference->deleteObjects);
+  if (info->root != &info->nil) {
+    DeleteSubTrees(info->root, reference->deleteObjects);
     delete info->root;
-    info->root = &Element::nil;
+    info->root = &info->nil;
     reference->size = 0;
   }
 }
@@ -891,15 +938,15 @@ PObject * PAbstractSortedList::GetAt(PINDEX index) const
   if (index != info->lastIndex) {
     if (index == info->lastIndex-1) {
       info->lastIndex--;
-      info->lastElement = info->lastElement->Predecessor();
+      info->lastElement = Predecessor(info->lastElement);
     }
     else if (index == info->lastIndex+1 && info->lastElement != NULL) {
       info->lastIndex++;
-      info->lastElement = info->lastElement->Successor();
+      info->lastElement = Successor(info->lastElement);
     }
     else {
       info->lastIndex = index;
-      info->lastElement = info->root->OrderSelect(index+1);
+      info->lastElement = OrderSelect(info->root, index+1);
     }
   }
 
@@ -910,7 +957,7 @@ PObject * PAbstractSortedList::GetAt(PINDEX index) const
 PINDEX PAbstractSortedList::GetObjectsIndex(const PObject * obj) const
 {
   Element * elmt = NULL;
-  PINDEX pos = info->root->ValueSelect(*obj, elmt);
+  PINDEX pos = ValueSelect(info->root, *obj, (const Element **)&elmt);
   if (pos == P_MAX_INDEX)
     return P_MAX_INDEX;
 
@@ -918,14 +965,14 @@ PINDEX PAbstractSortedList::GetObjectsIndex(const PObject * obj) const
     PINDEX savePos = pos;
     Element * saveElmt = elmt;
     while (elmt->data != obj &&
-            (elmt = elmt->Predecessor()) != &Element::nil &&
+            (elmt = Predecessor(elmt)) != &info->nil &&
             *obj == *elmt->data)
       pos--;
     if (elmt->data != obj) {
       pos = savePos;
       elmt = saveElmt;
       while (elmt->data != obj &&
-              (elmt = elmt->Successor()) != &Element::nil &&
+              (elmt = Successor(elmt)) != &info->nil &&
               *obj == *elmt->data)
         pos++;
       if (elmt->data != obj)
@@ -942,14 +989,14 @@ PINDEX PAbstractSortedList::GetObjectsIndex(const PObject * obj) const
 
 PINDEX PAbstractSortedList::GetValuesIndex(const PObject & obj) const
 {
-  PINDEX pos = info->root->ValueSelect(obj, info->lastElement);
+  PINDEX pos = ValueSelect(info->root, obj, (const Element **)&info->lastElement);
   if (pos == P_MAX_INDEX)
     return P_MAX_INDEX;
 
   info->lastIndex = pos;
 
   Element * prev;
-  while ((prev = info->lastElement->Predecessor()) != &Element::nil &&
+  while ((prev = Predecessor(info->lastElement)) != &info->nil &&
                                   prev->data->Compare(obj) == EqualTo) {
     info->lastElement = prev;
     info->lastIndex--;
@@ -964,24 +1011,24 @@ void PAbstractSortedList::RemoveElement(Element * node)
   PAssertNULL(node);
 
   // Don't try an remove one of the special leaf nodes!
-  if (node == &Element::nil)
+  if (node == &info->nil)
     return;
 
   if (node->data != NULL && reference->deleteObjects)
     delete node->data;
 
-  Element * y = node->left == &Element::nil || node->right == &Element::nil ? node : node->Successor();
+  Element * y = node->left == &info->nil || node->right == &info->nil ? node : Successor(node);
 
   Element * t = y;
-  while (t != &Element::nil) {
+  while (t != &info->nil) {
     t->subTreeSize--;
     t = t->parent;
   }
 
-  Element * x = y->left != &Element::nil ? y->left : y->right;
+  Element * x = y->left != &info->nil ? y->left : y->right;
   x->parent = y->parent;
 
-  if (y->parent == &Element::nil)
+  if (y->parent == &info->nil)
     info->root = x;
   else if (y == y->parent->left)
     y->parent->left = x;
@@ -1061,10 +1108,10 @@ void PAbstractSortedList::LeftRotate(Element * node)
 {
   Element * pivot = PAssertNULL(node)->right;
   node->right = pivot->left;
-  if (pivot->left != &Element::nil)
+  if (pivot->left != &info->nil)
     pivot->left->parent = node;
   pivot->parent = node->parent;
-  if (node->parent == &Element::nil)
+  if (node->parent == &info->nil)
     info->root = pivot;
   else if (node == node->parent->left)
     node->parent->left = pivot;
@@ -1081,10 +1128,10 @@ void PAbstractSortedList::RightRotate(Element * node)
 {
   Element * pivot = PAssertNULL(node)->left;
   node->left = pivot->right;
-  if (pivot->right != &Element::nil)
+  if (pivot->right != &info->nil)
     pivot->right->parent = node;
   pivot->parent = node->parent;
-  if (node->parent == &Element::nil)
+  if (node->parent == &info->nil)
     info->root = pivot;
   else if (node == node->parent->right)
     node->parent->right = pivot;
@@ -1097,27 +1144,17 @@ void PAbstractSortedList::RightRotate(Element * node)
 }
 
 
-PAbstractSortedList::Element::Element(PObject * theData)
-{
-  parent = left = right = &nil;
-  colour = Black;
-  subTreeSize = theData != NULL ? 1 : 0;
-  data = theData;
-}
-
-
-PAbstractSortedList::Element * PAbstractSortedList::Element::Successor() const
+PAbstractSortedList::Element * PAbstractSortedList::Successor(const Element * node) const
 {
   Element * next;
-  if (right != &nil) {
-    next = right;
-    while (next->left != &nil)
+  if (node->right != &info->nil) {
+    next = node->right;
+    while (next->left != &info->nil)
       next = next->left;
   }
   else {
-    next = parent;
-    const Element * node = this;
-    while (next != &nil && node == next->right) {
+    next = node->parent;
+    while (next != &info->nil && node == next->right) {
       node = next;
       next = node->parent;
     }
@@ -1126,18 +1163,17 @@ PAbstractSortedList::Element * PAbstractSortedList::Element::Successor() const
 }
 
 
-PAbstractSortedList::Element * PAbstractSortedList::Element::Predecessor() const
+PAbstractSortedList::Element * PAbstractSortedList::Predecessor(const Element * node) const
 {
   Element * pred;
-  if (left != &nil) {
-    pred = left;
-    while (pred->right != &nil)
+  if (node->left != &info->nil) {
+    pred = node->left;
+    while (pred->right != &info->nil)
       pred = pred->right;
   }
   else {
-    pred = parent;
-    const Element * node = this;
-    while (pred != &nil && node == pred->left) {
+    pred = node->parent;
+    while (pred != &info->nil && node == pred->left) {
       node = pred;
       pred = node->parent;
     }
@@ -1146,45 +1182,46 @@ PAbstractSortedList::Element * PAbstractSortedList::Element::Predecessor() const
 }
 
 
-PAbstractSortedList::Element * PAbstractSortedList::Element::OrderSelect(PINDEX index)
+PAbstractSortedList::Element * PAbstractSortedList::OrderSelect(Element * node, PINDEX index) const
 {
-  PINDEX r = left->subTreeSize+1;
+  PINDEX r = node->left->subTreeSize+1;
   if (index == r)
-    return this;
+    return node;
 
   if (index < r) {
-    if (left != &nil)
-      return left->OrderSelect(index);
+    if (node->left != &info->nil)
+      return OrderSelect(node->left, index);
   }
   else {
-    if (right != &nil)
-      return right->OrderSelect(index - r);
+    if (node->right != &info->nil)
+      return OrderSelect(node->right, index - r);
   }
 
   PAssertAlways2("PAbstractSortedList::Element", "Order select failed!");
-  return &nil;
+  return (Element *)&info->nil;
 }
 
 
-PINDEX PAbstractSortedList::Element::ValueSelect(const PObject & obj,
-                                                 Element * & lastElement)
+PINDEX PAbstractSortedList::ValueSelect(const Element * node,
+                                        const PObject & obj,
+                                        const Element ** lastElement) const
 {
-  if (this != &nil) {
-    switch (data->Compare(obj)) {
+  if (node != &info->nil) {
+    switch (node->data->Compare(obj)) {
       case PObject::LessThan :
       {
-        PINDEX index = right->ValueSelect(obj, lastElement);
+        PINDEX index = ValueSelect(node->right, obj, lastElement);
         if (index != P_MAX_INDEX)
-          return left->subTreeSize + index + 1;
+          return node->left->subTreeSize + index + 1;
         break;
       }
 
       case PObject::GreaterThan :
-        return left->ValueSelect(obj, lastElement);
+        return ValueSelect(node->left, obj, lastElement);
 
       default :
-        lastElement = this;
-        return left->subTreeSize;
+        *lastElement = node;
+        return node->left->subTreeSize;
     }
   }
 
@@ -1192,33 +1229,25 @@ PINDEX PAbstractSortedList::Element::ValueSelect(const PObject & obj,
 }
 
 
-void PAbstractSortedList::Element::DeleteSubTrees(BOOL deleteObject)
+void PAbstractSortedList::DeleteSubTrees(Element * node, BOOL deleteObject)
 {
-  if (left != &nil) {
-    left->DeleteSubTrees(deleteObject);
-    delete left;
-    left = &nil;
+  if (node->left != &info->nil) {
+    DeleteSubTrees(node->left, deleteObject);
+    delete node->left;
+    node->left = &info->nil;
   }
-  if (right != &nil) {
-    right->DeleteSubTrees(deleteObject);
-    delete right;
-    right = &nil;
+  if (node->right != &info->nil) {
+    DeleteSubTrees(node->right, deleteObject);
+    delete node->right;
+    node->right = &info->nil;
   }
   if (deleteObject) {
-    delete data;
-    data = NULL;
+    delete node->data;
+    node->data = NULL;
   }
 }
 
 
-PAbstractSortedList::Info::Info()
-{
-  root = &Element::nil;
-  lastElement = NULL;
-  lastIndex = P_MAX_INDEX;
-}
-
-
 ///////////////////////////////////////////////////////////////////////////////
 
 PObject * POrdinalKey::Clone() const
@@ -1672,8 +1701,8 @@ BOOL PAbstractDictionary::AbstractSetAt(const PObject & key, PObject * obj)
       reference->size++;
     }
     else {
-      if (reference->deleteObjects)
-        delete hashTable->lastElement->data;
+      if ((reference->deleteObjects) && (hashTable->lastElement->data != obj)) 
+	delete hashTable->lastElement->data;
       hashTable->lastElement->data = obj;
     }
   }
