@@ -24,6 +24,15 @@
  * Contributor(s): Derek J Smithies (derek@indranet.co.nz)
  *
  * $Log: vfakeio.cxx,v $
+ * Revision 1.29  2004/03/17 22:24:29  rjongbloed
+ * Fixed MSVC warnings
+ *
+ * Revision 1.28  2004/03/16 10:10:47  csoutheren
+ * Proper fix that does not generate warnings
+ *
+ * Revision 1.27  2004/03/16 08:27:14  csoutheren
+ * Removed illegal characters
+ *
  * Revision 1.26  2003/12/14 10:01:02  rjongbloed
  * Resolved issue with name space conflict os static and virtual forms of GetDeviceNames() function.
  *
@@ -430,7 +439,7 @@ static OneVFakeLetterData vFakeLetterData[] = {
 	     "******",
              "      ",
              "      " }},
-    { '', { "      ",
+    { '\xe6', { "      ",
 	     "      ",
 	     "      ",
 	     " ** * ",
@@ -441,7 +450,7 @@ static OneVFakeLetterData vFakeLetterData[] = {
 	     " ** * ",
              "      ",
              "      " }},
-    { '', { "      ",
+    { '\xf8', { "      ",
 	     "      ",
 	     "     *",
 	     " **** ",
@@ -452,7 +461,7 @@ static OneVFakeLetterData vFakeLetterData[] = {
 	     " **** ",
              "*     ",
              "      " }},
-    { '', { "  **  ",
+    { '\xe5', { "  **  ",
 	     " *  * ",
 	     "  **  ",
 	     " **** ",
@@ -749,7 +758,7 @@ static OneVFakeLetterData vFakeLetterData[] = {
 	     "*****",
              "     ",
              "     " }},
-    { '', { " ** ***",
+    { '\xc6', { " ** ***",
 	     "*  *   ",
 	     "*  *   ",
 	     "*  *   ",
@@ -760,7 +769,7 @@ static OneVFakeLetterData vFakeLetterData[] = {
 	     "*  ****",
              "       ",
              "       " }},
-    { '', { " *** ",
+    { '\xd8', { " *** ",
 	     "*  **",
 	     "* * *",
 	     "* * *",
@@ -771,7 +780,7 @@ static OneVFakeLetterData vFakeLetterData[] = {
 	     " *** ",
              "*    ",
              "     " }},
-    { '', { "  **  ",
+    { '\xc5', { "  **  ",
 	     " *  * ",
 	     "  **  ",
 	     "  **  ",
