@@ -25,10 +25,17 @@
 #ifndef _PWLib_VERSION_H
 #define _PWLib_VERSION_H
 
+
+
+/* WARNING: You MUST NOT add any comments to the #defines which follow
+   or add extra commented out #defines as this will confuse the parser
+   which extracts the version numbers
+ */
+
 #define MAJOR_VERSION 1
 #define MINOR_VERSION 6
 #define BUILD_TYPE    ReleaseCode
-#define BUILD_NUMBER 3
+#define BUILD_NUMBER 5
 
 
 #endif  // _PWLib_VERSION_H
